"""
This work has little to do with Polylidar3D. Just some work demonstrating mesh creation using different techniques
"""
import time
import math

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

from polylidar import Polylidar3D, MatrixDouble
from polylidar.polylidarutil import (plot_polygons_3d, generate_3d_plane, set_axes_equal, plot_planes_3d,
                           scale_points, rotation_matrix, apply_rotation, COLOR_PALETTE)

from scipy.spatial import Delaunay
from scipy.stats import describe
import open3d as o3d

def create_open_3d_mesh(triangles, points, color=[1, 0, 0]):
    mesh_2d = o3d.geometry.TriangleMesh()
    mesh_2d.triangles = o3d.utility.Vector3iVector(triangles)
    mesh_2d.vertices = o3d.utility.Vector3dVector(points)
    mesh_2d.compute_vertex_normals()
    mesh_2d.compute_triangle_normals()
    mesh_2d.paint_uniform_color(color)
    return mesh_2d

def extract_mesh_planes(points, mesh, planes, color=None):
    triangles = np.asarray(mesh.triangles)
    meshes = []
    for i, plane in enumerate(planes):
        if color is None:
            color = COLOR_PALETTE[i]
        tris = np.ascontiguousarray(np.flip(triangles[plane, :], 1))
        mesh = create_open_3d_mesh(tris, points, color)
        meshes.append(mesh)
    return meshes


def generate_point_cloud(max_size=10):
    np.random.seed(1)
    # generate random plane with hole
    plane = generate_3d_plane(bounds_x=[0, max_size, 0.5], bounds_y=[0, max_size, 0.5], holes=[], #holes=[[[3, 5], [3, 5]]], 
                            height_noise=0.05, planar_noise=0.02)
    # Add double plane to simulate extra noise
    # plane2 = plane + [0.1, 0.1, 0.05]
    # plane = np.vstack((plane, plane2))
    # Generate top of box (causing the hole that we see)
    box_top = generate_3d_plane(bounds_x=[3, 5, 0.2], bounds_y=[3, 5, 0.2], holes=[
    ], height_noise=0.02, height=2, planar_noise=0.02)
    # Generate side of box (causing the hole that we see)
    box_side = generate_3d_plane(bounds_x=[0, 2, 0.2], bounds_y=[
                                0, 2, 0.2], holes=[], height_noise=0.02, planar_noise=0.02)
    rm = rotation_matrix([0,1,0], -math.pi/2.0)
    box_side = apply_rotation(rm, box_side) + [5, 3, 0]
    # box_side = r.apply(box_side) + [5, 3, 0]
    # All points joined together
    points = np.ascontiguousarray(np.concatenate((plane, box_side, box_top)))
    return points

def create_open3d_pc(points):
    pcd = o3d.geometry.PointCloud()
    pcd.points = o3d.utility.Vector3dVector(points)
    return pcd

def main():
    points = generate_point_cloud()
    points_mat = MatrixDouble(points)
    polylidar_kwargs = dict(alpha=0.0, lmax=1.0, min_triangles=20, z_thresh=0.2, norm_thresh=0.98)
    polylidar = Polylidar3D(**polylidar_kwargs)
    t1 = time.perf_counter()
    # Create Pseudo 3D Surface Mesh using Delaunay Triangulation and Polylidar
    mesh, planes, polygons = polylidar.extract_planes_and_polygons(points_mat)
    t2 = time.perf_counter()
    print("Point Size: {}".format(points.shape[0]))
    print("2.5D Delaunay Triangulation and Plane Extraction; Mesh Creation {:.2f} milliseconds".format((t2 - t1) * 1000))

    # Visualize
    # Create Open3D Mesh
    mesh_planes = extract_mesh_planes(points, mesh, planes, COLOR_PALETTE[0])
    # Create Open 3D Point Cloud
    pcd = create_open3d_pc(points)
    o3d.visualization.draw_geometries([pcd, *mesh_planes])

    # Estimate Point Cloud Normals
    t0 = time.perf_counter()
    pcd.estimate_normals(search_param=o3d.geometry.KDTreeSearchParamHybrid(radius=1.0, max_nn=8))
    t1 = time.perf_counter()
    # Create True 3D Surface Mesh using Ball Pivot Algorithm
    radii = o3d.utility.DoubleVector([0.4, 0.5])
    mesh = o3d.geometry.TriangleMesh.create_from_point_cloud_ball_pivoting(pcd, radii)
    mesh.paint_uniform_color(COLOR_PALETTE[0])
    t2 = time.perf_counter()
    print("3D Triangulation using Ball Pivot: Normal Estimation took: {:.2f}, Mesh Creation: {:.2f}".format((t1-t0) * 1000,(t2-t1) * 1000))
    # Visualize
    o3d.visualization.draw_geometries([pcd, mesh])

    # Create True 3D Surface Mesh using Poisson Reconstruction Algorithm
    t3 = time.perf_counter()
    mesh, densities = o3d.geometry.TriangleMesh. create_from_point_cloud_poisson(pcd, depth=6)
    t4 = time.perf_counter()
    print("3D Triangulation using Poisson Reconstruction: Mesh Creation: {:.2f}".format((t4-t3) * 1000))
    # Visualize
    # o3d.visualization.draw_geometries([mesh])
    # print(mesh)
    # print('visualize densities')
    densities = np.asarray(densities)
    density_colors = plt.get_cmap('plasma')(
        (densities - densities.min()) / (densities.max() - densities.min()))
    density_colors = density_colors[:, :3]
    density_mesh = o3d.geometry.TriangleMesh()
    density_mesh.vertices = mesh.vertices
    density_mesh.triangles = mesh.triangles
    density_mesh.triangle_normals = mesh.triangle_normals
    density_mesh.vertex_colors = o3d.utility.Vector3dVector(density_colors)
    # mesh.vertex_colors = o3d.utility.Vector3dVector(density_colors)
    # o3d.visualization.draw_geometries([density_mesh])

    # print('remove low density vertices')
    vertices_to_remove = densities < np.quantile(densities, 0.1)
    mesh.remove_vertices_by_mask(vertices_to_remove)
    mesh.compute_triangle_normals()
    mesh.paint_uniform_color(COLOR_PALETTE[0])
    # print(mesh)
    o3d.visualization.draw_geometries([pcd, mesh])


if __name__ == "__main__":
    main()




"""Used to test Polyidars Matrix Class
This test ensures that a no copy operation actually works
"""
import numpy as np
from polylidar import Polylidar3D, MatrixDouble

def get_np_buffer_ptr(a):
    pointer, read_only_flag = a.__array_interface__['data']
    return pointer

def main():
    a = np.arange(4, dtype=np.float64).reshape((2,2))
    print("Numpy Array and Buffer Ptr")
    print(a)
    print(get_np_buffer_ptr(a))
    b = MatrixDouble(a, False)
    c = np.asarray(b)
    print("")
    print("Matrix Array and Buffer Ptr")
    print(c)
    print(get_np_buffer_ptr(c))
    try:

        d = a.reshape((4,))
        print("")
        print("Reshape to 1 Dimension")
        print(d)
        print("Should throw runtime error because not 2 Dim")
        e = MatrixDouble(d, False)
    except Exception:
        print("Successfully threw an exception")



if __name__ == "__main__":
    main()


import time
import math

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib
from mpl_toolkits.mplot3d import Axes3D
from shapely.geometry import Polygon, MultiPolygon
from descartes import PolygonPatch


from polylidar import Delaunator, MatrixDouble, Polylidar3D
from polylidar.polylidarutil import (plot_polygons_3d, generate_3d_plane, set_axes_equal,
                           scale_points, rotation_matrix, apply_rotation, get_point)
from polylidar.polylidarutil.plane_filtering import get_points

COLOR_PALETTE = list(map(colors.to_rgb, plt.rcParams['axes.prop_cycle'].by_key()['color']))


matplotlib.rc('xtick', labelsize=18) 
matplotlib.rc('ytick', labelsize=18)
# matplotlib.rc('ztick', labelsize=20)
matplotlib.rc('axes', labelsize=18) 


def plot_polygons(polygons, points, ax, shell_color='green', hole_color='orange'):
    for poly in polygons:
        shell_coords = [get_point(pi, points) for pi in poly.shell]
        outline = Polygon(shell=shell_coords)
        outlinePatch = PolygonPatch(outline, ec=shell_color, fill=False, linewidth=3, zorder=4)
        ax.add_patch(outlinePatch)

        for hole_poly in poly.holes:
            shell_coords = [get_point(pi, points) for pi in hole_poly]
            outline = Polygon(shell=shell_coords)
            outlinePatch = PolygonPatch(outline, ec=hole_color, fill=False, linewidth=3, zorder=4)
            ax.add_patch(outlinePatch)

def plot_planes_3d(lidar_building, triangles, planes, ax, alpha=[1.0], z_value=None, z_scale=1.0, color=None):
    ax_planes = []
    for i, plane in enumerate(planes):
        triangles_plane = triangles[plane]
        color_ = COLOR_PALETTE[i] if color is None else color
        color_ = color_ + (alpha[i], )
        ax_plane = ax.plot_trisurf(*scale_points(lidar_building, z_value=z_value, z_scale=z_scale),triangles=triangles_plane, color=color_,  edgecolor=(0,0,0,0.3), linewidth=0.5)
        ax_planes.append(ax_plane)
    return ax_planes

def main():
    # np.random.seed(5)
    np.random.seed(9)
    # generate random plane with hole
    plane = generate_3d_plane(bounds_x=[0, 10, 0.5], bounds_y=[0, 10, 0.5], holes=[
                            [[3, 5], [3, 5]]], height_noise=0.05, planar_noise=0.10)
    # Generate top of box (causing the hole that we see)
    box_top = generate_3d_plane(bounds_x=[3, 5, 0.2], bounds_y=[3, 5, 0.2], holes=[
    ], height_noise=0.02, height=2, planar_noise=0.02)
    # Generate side of box (causing the hole that we see)
    box_side = generate_3d_plane(bounds_x=[0, 2, 0.2], bounds_y=[
                                0, 2, 0.2], holes=[], height_noise=0.02, planar_noise=0.02)
    rm = rotation_matrix([0,1,0], -math.pi/2.0)
    box_side = apply_rotation(rm, box_side) + [5, 3, 0]
    # box_side = r.apply(box_side) + [5, 3, 0]
    # All points joined together
    points = np.concatenate((plane, box_side, box_top))

    points_mat = MatrixDouble(points)
    polylidar_kwargs = dict(alpha=0.0, lmax=1.0, min_triangles=20, z_thresh=0.1, norm_thresh_min=0.94)
    polylidar = Polylidar3D(**polylidar_kwargs)

    elev = 15.0
    azim = -125


    # Extracts planes and polygons, time
    t1 = time.time()
    mesh, planes, polygons = polylidar.extract_planes_and_polygons(points_mat)
    t2 = time.time()
    print("Polylidar Took {:.2f} milliseconds".format((t2 - t1) * 1000))

    triangles = np.asarray(mesh.triangles)
    all_planes = [np.arange(triangles.shape[0])]
    
    print("")
    print("Should see a mesh segments")
    fig, ax = plt.subplots(figsize=(10, 10), nrows=1, ncols=1,
                        subplot_kw=dict(projection='3d'))
    # plot all triangles
    # plot_polygons_3d(points, polygons, ax)
    rm_45 = rotation_matrix([0,1,0], -math.pi/4.0)
    points_rot = apply_rotation(rm_45, points)
    plot_planes_3d(points_rot, triangles, [planes[1]], ax, alpha=[0.85])
    # plot points
    # ax.scatter(*scale_points(points), c='k', s=0.1)
    set_axes_equal(ax)
    ax.view_init(elev=elev, azim=azim)
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')
    fig.savefig("assets/scratch/PolygonExtraction_a.pdf", bbox_inches='tight')
    # fig.savefig("assets/scratch/Basic25DAlgorithm_polygons.png", bbox_inches='tight', pad_inches=-0.8)
    plt.show()

    # Show 2D Projection and Polygon Extraction
    print("")
    print("Should see projected vertices to geometric plane")
    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=(5,5))
    simplices_plane = triangles[planes[1], :]
    plot_polygons([polygons[1]], points[:, :2], ax, shell_color=COLOR_PALETTE[4], hole_color=COLOR_PALETTE[4] )
    ax.triplot(points[:,0], points[:,1], simplices_plane, c='k', alpha=0.75)
    xlim = ax.get_xlim()
    ylim = ax.get_ylim()
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    fig.savefig("assets/scratch/PolygonExtraction_b1.pdf", bbox_inches='tight')
    plt.show()

    # Show 2D Projection and Polygon Extraction
    print("")
    print("Should see projected vertices to geometric plane")
    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=(5,5))
    simplices_plane = triangles[planes[1], :]
    plot_polygons([polygons[1]], points[:, :2], ax )
    ax.triplot(points[:,0], points[:,1], simplices_plane, c='k', alpha=0.75)
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    fig.savefig("assets/scratch/PolygonExtraction_b2.pdf", bbox_inches='tight')
    plt.show()

    poly = polygons[1]
    shell_coords = get_points(poly.shell, points)
    hole_coords = [get_points(hole, points) for hole in poly.holes]
    poly_shape = Polygon(shell=shell_coords, holes=hole_coords)

    # Show Simplification 
    print("")
    print("Should see simplified Polygon")
    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=(5,5))

    patch = PolygonPatch(poly_shape, fill=True, alpha=0.5)
    ax.add_patch(patch)
    # plot_polygons([polygons[1]], points[:, :2], ax)
    poly_simplify = poly_shape.simplify(.2, preserve_topology=True)
    patch = PolygonPatch(poly_simplify, color='k', fill=False, linewidth=1.5, linestyle='--', zorder=1)
    ax.add_patch(patch)
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    print(f"Before Vertices: {len(poly_shape.exterior.coords)}")
    print(f"After Vertices: {len(poly_simplify.exterior.coords)}")
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    fig.savefig("assets/scratch/PolygonExtraction_c.pdf", bbox_inches='tight')
    plt.show()

    # Show Positive Buffer
    print("")
    print("Should see positive buffer Polygon")
    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=(5,5))

    patch = PolygonPatch(poly_shape, fill=True, alpha=0.5)
    ax.add_patch(patch)
    # plot_polygons([polygons[1]], points[:, :2], ax)
    poly_buffer = poly_simplify.buffer(.2)
    patch = PolygonPatch(poly_buffer, color='k', fill=False, linewidth=1.5, linestyle='--', zorder=1)
    ax.add_patch(patch)
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    print(f"After Vertices: {len(poly_buffer.exterior.coords)}")
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    fig.savefig("assets/scratch/PolygonExtraction_d.pdf", bbox_inches='tight')
    plt.show()

    # Show Negative Buffer
    print("")
    print("Should see negative buffer Polygon")
    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=(5,5))

    patch = PolygonPatch(poly_shape, fill=True, alpha=0.5)
    ax.add_patch(patch)
    # plot_polygons([polygons[1]], points[:, :2], ax)
    poly_buffer = poly_buffer.buffer(-.2)
    patch = PolygonPatch(poly_buffer, color='k', fill=False, linewidth=1.5, linestyle='--', zorder=1)
    ax.add_patch(patch)
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    print(f"After Vertices: {len(poly_buffer.exterior.coords)}")
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    fig.savefig("assets/scratch/PolygonExtraction_e.pdf", bbox_inches='tight')
    plt.show()



if __name__ == "__main__":
    main()



import time

import numpy as np
from scipy import spatial
from scipy.spatial.transform import Rotation as R
from shapely.geometry import Polygon, JOIN_STYLE
import logging

IDENTITY = R.identity()
logging.basicConfig(level=logging.INFO)

import matplotlib.pyplot as plt


def plot_indices_text(indices, points, ax):
    for i in range(indices.shape[0]):
        ax.text(points[i, 0], points[i, 1], str(indices[i]))


def plot_poly(polygon: Polygon, ax, polygon_indices=None):
    from shapely.geometry import Polygon
    from descartes import PolygonPatch
    outlinePatch = PolygonPatch(Polygon(polygon.exterior), ec='green', fill=False, linewidth=2)
    for i, hole in enumerate(polygon.interiors):
        color = plt.cm.get_cmap('tab20')([i])[0][:3]
        pl = Polygon(shell=hole)
        pp = PolygonPatch(pl, fill=False, linewidth=2, ec=color)
        ax.add_patch(pp)
    if polygon_indices:
        lr_indices = np.asarray(polygon_indices.shell)
        lr_points = np.array(polygon.exterior)
        plot_indices_text(lr_indices, lr_points, ax)
        for i, hole in enumerate(polygon.interiors):
            lr_indices = np.asarray(polygon_indices.holes[i])
            lr_points = np.asarray(hole)
            plot_indices_text(lr_indices, lr_points, ax)

    ax.add_patch(outlinePatch)


def get_points(point_idxs, points):
    return points[point_idxs, :]


def create_kd_tree(shell_coords, hole_coords):
    hole_coords.append(shell_coords)
    all_vertices = np.vstack(hole_coords)
    logging.debug("KD Tree Size: %s", all_vertices.shape)
    kd_tree = spatial.cKDTree(all_vertices, leafsize=10)
    return kd_tree


def add_column(array, z_value):
    ones = np.ones((array.shape[0], 1)) * z_value
    stacked = np.column_stack((array, ones))
    return stacked


def recover_3d(poly, kd_tree, z_value):
    points = np.asarray(poly.exterior)
    if points.shape[1] == 3:
        return poly
    shell_3D = add_column(points, z_value)
    # print(shell_3D.shape)
    t0 = time.perf_counter()
    d, shell_idx = kd_tree.query(shell_3D)
    t1 = time.perf_counter()
    # print(shell_idx.shape)
    kd_data = kd_tree.data[shell_idx, :]
    # print(kd_data.shape)
    shell_3D[:, 2] = kd_data[:, 2]
    holes_lr = []
    t2 = time.perf_counter()
    for hole in poly.interiors:
        hole_lr = add_column(np.array(hole), z_value)
        d, shell_idx = kd_tree.query(hole_lr)
        kd_data = kd_tree.data[shell_idx, :]
        hole_lr[:, 2] = kd_data[:, 2]
        holes_lr.append(hole_lr)
    t3 = time.perf_counter()

    # print("Shell Length: {}; Query KD Tree Shell: {:.2f}; Query KD Tree Holes: {:.2f}".format(
    #     shell_3D.shape[0], (t1-t0) * 1000, (t3-t2) * 1000
    # ))

    poly_3d = Polygon(shell=shell_3D, holes=holes_lr)
    return poly_3d


def filter_planes_and_holes(polygons, points, config_pp, rm=None):
    """Extracts the plane and obstacles returned from polylidar.
    This function performs post-processing of the Polygons returned by Polylidar3D using the Shapely library.
    If the polygons are 3D you must provide a scipy rotation matrix such that the polygon
    align with they XY plane (Shapely can only handle 2D polygons with XY coordinates).

    The basic steps are:
        * Simplification of Polygon by config_pp['simplify]
        * Positive buffer of Polygon by config_pp['positive_buffer]
        * Negative Buffer of POlygons by config_pp['negative_buffer]
        * Simplification of Polygon by config_pp['simplify]
        * Remove polygons whose area is less or greater than data in config_pp['filter]['plane_area']
        * Remove holes whose vertices are less than data in config_pp['filter]['hole_vertices']
        * Remove holes whose area is less or greater than data in config_pp['filter]['hole_area']

    It then returns the shapely polygons of the polygons and holes (obstacles)

    An example config_pp

    .. code-block:: python

        {
            positive_buffer: 0.005 # m, Positively expand polygon.  Fills in small holes
            negative_buffer: 0.03 # m, Negative buffer to polygon. Expands holes and constricts outer hull of polygon
            simplify: 0.02  # m, simplify edges of polygon
            filter: # obstacles must have these characteristics
                hole_area:
                    min: 0.025   # m^2
                    max: 0.785 # m^2
                hole_vertices:
                    min: 6
                plane_area:
                    min: .5 # m^2
        }

    Arguments:
        polygons {list[Polygons]} -- A list of polygons returned from polylidar
        points {ndarray} -- MX3 array
        config_pp {dict} -- Configuration for post processing filtering
        rm {scipy.spatial.RotationMatrix} -- Rotation matrix applied to 3D polygons to make 2D

    Returns:
        tuple -- A list of plane shapely polygons and a list of holes in polygons
    """
    # filtering configuration
    post_filter = config_pp['filter']

    # will hold the plane(s) and obstacles found
    planes = []
    obstacles = []
    # print("Polylidar returned {} polygons, ".format(len(polygons)))
    for poly in polygons:
        t0 = time.perf_counter()
        if rm is not None:
            shell_coords = rm.apply(get_points(poly.shell, points))
            hole_coords = [rm.apply(get_points(hole, points)) for hole in poly.holes]
        else:
            shell_coords = get_points(poly.shell, points)
            hole_coords = [get_points(hole, points) for hole in poly.holes]
        t1 = time.perf_counter()
        poly_shape = Polygon(shell=shell_coords, holes=hole_coords)
        t2 = time.perf_counter()
        # print(poly_shape.is_valid)
        # fig, ax = plt.subplots(figsize=(10, 10), nrows=1, ncols=1)
        # plot_poly(poly_shape, ax, poly)
        # plt.axis('equal')
        # plt.show()
        # print(poly_shape.is_valid)
        # assert poly_shape.is_valid
        area = poly_shape.area
        # logging.info("Got a plane!")
        if post_filter['plane_area']['min'] and area < post_filter['plane_area']['min']:
            # logging.info("Skipping Plane")
            continue
        z_value = shell_coords[0][2]

        t3 = time.perf_counter()
        if config_pp['simplify']:
            poly_shape = poly_shape.simplify(
                tolerance=config_pp['simplify'], preserve_topology=True)
        t4 = time.perf_counter()
        # Perform 2D geometric operations
        if config_pp['positive_buffer']:
            poly_shape = poly_shape.buffer(
                config_pp['positive_buffer'], join_style=JOIN_STYLE.mitre, resolution=4)
        t5 = time.perf_counter()
        if config_pp['negative_buffer']:
            poly_shape = poly_shape.buffer(
                distance=-config_pp['negative_buffer'], join_style=JOIN_STYLE.mitre, resolution=4)
            # if poly_shape.geom_type == 'MultiPolygon':
            #     all_poly_shapes = list(poly_shape.geoms)
            #     poly_shape = sorted(
            #         all_poly_shapes, key=lambda geom: geom.area, reverse=True)[0]
        t6 = time.perf_counter()
        # poly_shape = poly_shape.buffer(distance=config_pp['negative_buffer'], resolution=4)
        if config_pp['simplify']:
            poly_shape = poly_shape.simplify(
                tolerance=config_pp['simplify'], preserve_topology=True)  # False makes fast, but can cause invalid polygons
        t7 = time.perf_counter()
        if poly_shape.geom_type == 'MultiPolygon':
            all_poly_shapes = list(poly_shape.geoms)
            # poly_shape = sorted(
            #     all_poly_shapes, key=lambda geom: geom.area, reverse=True)[0]
        else:
            all_poly_shapes = [poly_shape]

        logging.debug("Rotation: {:.2f}; Polygon Creation: {:.2f}; Simplify 1: {:.2f}; Positive Buffer: {:.2f}; Negative Buffer: {:.2f}; Simplify 2: {:.2f}".format(
            (t1 - t0) * 1000, (t2 - t1) * 1000, (t4 - t3) * 1000, (t5 - t4) * 1000, (t6 - t5) * 1000, (t7 - t6) * 1000
        ))

        # Its possible that our polygon has no broken into a multipolygon
        # Check for this situation and handle it
        # all_poly_shapes = [poly_shape]
        # print(len(all_poly_shapes))
        # iterate through every polygons and check for plane extraction
        for poly_shape in all_poly_shapes:
            area = poly_shape.area
            # print(poly_shape.geom_type, area)
            # logging.info("Plane is big enough still")
            if post_filter['plane_area']['min'] <= 0 or area >= post_filter['plane_area']['min']:
                dim = np.asarray(poly_shape.exterior).shape[1]
                # logging.info("Plane is big enough still")
                if config_pp['negative_buffer'] or config_pp['simplify'] or config_pp['positive_buffer'] and dim < 3:
                    # convert back to 3D coordinates
                    # create kd tree for vertex lookup after buffering operations
                    t8 = time.perf_counter()
                    kd_tree = create_kd_tree(shell_coords, hole_coords)
                    t9 = time.perf_counter()
                    poly_shape = recover_3d(poly_shape, kd_tree, z_value)
                    t10 = time.perf_counter()
                    logging.debug("Create KD Tree: {:.2f}; Recover Polygon 3D Coordinates: {:.2f}".format(
                        (t9 - t8) * 1000, (t10 - t9) * 1000
                    ))

                # Capture the polygon as well as its z height
                # after applying buffering and simplification with shapely/geos all polygons are valid
                # print(poly_shape.is_valid)
                new_plane_polygon = Polygon(shell=poly_shape.exterior)
                planes.append((new_plane_polygon, z_value))

                for hole_lr in poly_shape.interiors:
                    # Filter by number of obstacle vertices, removes noisy holes
                    if len(hole_lr.coords) > post_filter['hole_vertices']['min']:
                        hole_poly = Polygon(shell=hole_lr)
                        area = hole_poly.area
                        # filter by area
                        if post_filter['hole_area']['min'] <= 0.0 or area >= post_filter['hole_area']['min'] and area < post_filter['hole_area']['max']:
                            z_value = hole_lr.coords[0][2]
                            obstacles.append((hole_poly, z_value))
    if rm is not None:
        t11 = time.perf_counter()
        rm_inv = rm.inv()
        for i, (poly, z_value) in enumerate(planes):
            points = np.asarray(poly.exterior)
            new_poly = Polygon(rm_inv.apply(points))
            planes[i] = (new_poly, z_value)

        for i, (poly, z_value) in enumerate(obstacles):
            points = np.asarray(poly.exterior)
            new_poly = Polygon(rm_inv.apply(points))
            obstacles[i] = (new_poly, z_value)
        t12 = time.perf_counter()
        logging.debug("Revert Rotation and Create New Polygons: {:2f}".format((t12 - t11) * 1000))
    return planes, obstacles


def filter_planes(polygons, points, config_pp, rm=None):
    """Converts polygons from Polylidar to Shapely polygons.
    This function performs post-processing of the Polygons returned by Polylidar3D using the Shapely library.
    If the polygons are 3D you must provide a scipy rotation matrix such that the polygon
    align with they XY plane (Shapely can only handle 2D polygons with XY coordinates).

    The basic steps are:
        * Simplification of Polygon by config_pp['simplify]
        * Positive buffer of Polygon by config_pp['positive_buffer]
        * Negative Buffer of POlygons by config_pp['negative_buffer]
        * Simplification of Polygon by config_pp['simplify]
        * Remove polygons whose area is less or greater than data in config_pp['filter]['plane_area']
        * Remove holes whose vertices are less than data in config_pp['filter]['hole_vertices']
        * Remove holes whose area is less or greater than data in config_pp['filter]['hole_area']

    It then returns the shapely polygons of the polygons and holes (obstacles)

    An example config_pp

    .. code-block:: python

        {
            positive_buffer: 0.005 # m, Positively expand polygon.  Fills in small holes
            negative_buffer: 0.03 # m, Negative buffer to polygon. Expands holes and constricts outer hull of polygon
            simplify: 0.02  # m, simplify edges of polygon
            filter: # obstacles must have these characteristics
                hole_area:
                    min: 0.025   # m^2
                    max: 0.785 # m^2
                hole_vertices:
                    min: 6
                plane_area:
                    min: .5 # m^2
        }

    Arguments:
        polygons {list[Polygons]} -- A list of polygons returned from polylidar
        points {ndarray} -- MX3 array
        config_pp {dict} -- Configuration for post processing filtering
        rm {scipy.spatial.RotationMatrix} -- Rotation matrix applied to 3D polygons to make 2D

    Returns:
        List -- A list of planar shapely polygons
    """
    # filtering configuration
    post_filter = config_pp['filter']

    # will hold the plane(s) and obstacles found
    planes = []
    # print("Polylidar returned {} polygons, ".format(len(polygons)))
    for poly in polygons:
        t0 = time.perf_counter()
        if rm is not None:
            shell_coords = rm.apply(get_points(poly.shell, points))
            hole_coords = [rm.apply(get_points(hole, points)) for hole in poly.holes]
        else:
            shell_coords = get_points(poly.shell, points)
            hole_coords = [get_points(hole, points) for hole in poly.holes]
        t1 = time.perf_counter()
        poly_shape = Polygon(shell=shell_coords, holes=hole_coords)
        t2 = time.perf_counter()
        # print("Before: ", poly_shape.is_valid)
        # fig, ax = plt.subplots(figsize=(10, 10), nrows=1, ncols=1)
        # plot_poly(poly_shape, ax, poly)
        # plt.axis('equal')
        # plt.show()
        # print(poly_shape.is_valid)
        # assert poly_shape.is_valid
        area = poly_shape.area
        # logging.info("Got a plane!")
        if post_filter['plane_area']['min'] and area < post_filter['plane_area']['min']:
            # logging.info("Skipping Plane")
            continue
        z_value = shell_coords[0][2]

        t3 = time.perf_counter()
        if config_pp['simplify']:
            poly_shape = poly_shape.simplify(
                tolerance=config_pp['simplify'], preserve_topology=True)
        t4 = time.perf_counter()
        # Perform 2D geometric operations
        if config_pp['positive_buffer']:
            poly_shape = poly_shape.buffer(
                config_pp['positive_buffer'], join_style=JOIN_STYLE.mitre, resolution=4)
        t5 = time.perf_counter()
        if config_pp['negative_buffer']:
            poly_shape = poly_shape.buffer(
                distance=-config_pp['negative_buffer'], join_style=JOIN_STYLE.mitre, resolution=4)
            # if poly_shape.geom_type == 'MultiPolygon':
            #     all_poly_shapes = list(poly_shape.geoms)
            #     poly_shape = sorted(
            #         all_poly_shapes, key=lambda geom: geom.area, reverse=True)[0]
        t6 = time.perf_counter()
        # poly_shape = poly_shape.buffer(distance=config_pp['negative_buffer'], resolution=4)
        if config_pp['simplify']:
            poly_shape = poly_shape.simplify(
                tolerance=config_pp['simplify'], preserve_topology=True)  # False makes fast, but can cause invalid polygons
        t7 = time.perf_counter()
        if poly_shape.geom_type == 'MultiPolygon':
            all_poly_shapes = list(poly_shape.geoms)
            # poly_shape = sorted(
            #     all_poly_shapes, key=lambda geom: geom.area, reverse=True)[0]
        else:
            all_poly_shapes = [poly_shape]

        logging.debug("Rotation: {:.2f}; Polygon Creation: {:.2f}; Simplify 1: {:.2f}; Positive Buffer: {:.2f}; Negative Buffer: {:.2f}; Simplify 2: {:.2f}".format(
            (t1 - t0) * 1000, (t2 - t1) * 1000, (t4 - t3) * 1000, (t5 - t4) * 1000, (t6 - t5) * 1000, (t7 - t6) * 1000
        ))

        # Its possible that our polygon has no broken into a multipolygon
        # Check for this situation and handle it
        # all_poly_shapes = [poly_shape]
        # print(len(all_poly_shapes))
        # iterate through every polygons and check for plane extraction
        for poly_shape in all_poly_shapes:
            area = poly_shape.area
            # print(poly_shape.geom_type, area)
            # logging.info("Plane is big enough still")
            if post_filter['plane_area']['min'] <= 0 or area >= post_filter['plane_area']['min']:
                dim = np.asarray(poly_shape.exterior).shape[1]
                # logging.info("Plane is big enough still")
                if config_pp['negative_buffer'] or config_pp['simplify'] or config_pp['positive_buffer'] and dim < 3:
                    # convert back to 3D coordinates
                    # create kd tree for vertex lookup after buffering operations
                    t8 = time.perf_counter()
                    kd_tree = create_kd_tree(shell_coords, hole_coords)
                    t9 = time.perf_counter()
                    poly_shape = recover_3d(poly_shape, kd_tree, z_value)
                    t10 = time.perf_counter()
                    logging.debug("Create KD Tree: {:.2f}; Recover Polygon 3D Coordinates: {:.2f}".format(
                        (t9 - t8) * 1000, (t10 - t9) * 1000
                    ))

                # Capture the polygon as well as its z height
                # after applying buffering and simplification with shapely/geos all polygons are valid
                # print(poly_shape.is_valid)
                valid_shell = poly_shape.exterior
                valid_holes = []

                for hole_lr in poly_shape.interiors:
                    # Filter by number of obstacle vertices, removes noisy holes
                    if len(hole_lr.coords) > post_filter['hole_vertices']['min']:
                        hole_poly = Polygon(shell=hole_lr)
                        area = hole_poly.area
                        # filter by area
                        if post_filter['hole_area']['min'] <= 0.0 or area >= post_filter['hole_area']['min'] and area < post_filter['hole_area']['max']:
                            valid_holes.append(hole_lr)

                new_plane_polygon = Polygon(shell=poly_shape.exterior, holes=valid_holes)
                planes.append((new_plane_polygon, z_value))
    
    if rm is not None:
        t11 = time.perf_counter()
        rm_inv = rm.inv()
        for i, (poly, z_value) in enumerate(planes):
            shell_points = rm_inv.apply(np.asarray(poly.exterior))
            hole_points = [rm_inv.apply(np.asarray(hole_lr)) for hole_lr in poly.interiors]
            new_poly = Polygon(shell=shell_points, holes=hole_points)
            planes[i] = (new_poly, z_value)
        t12 = time.perf_counter()
        logging.debug("Revert Rotation and Create New Polygons: {:2f}".format((t12 - t11) * 1000))
    return planes



"""Demo of Open3D 0.10.0 Slowdown
Please modify DIRECTORY to point to the folder of meshes attached to this issue reply
"""
import os
import open3d as o3d
DIRECTORY = 'o3d_slow_down'
def main():
    all_meshes = []
    all_files = sorted(list(os.listdir(DIRECTORY)))
    for filename in all_files:
        print(filename)
        mesh = o3d.io.read_triangle_mesh(os.path.join(DIRECTORY, filename))
        all_meshes.append(mesh)
    o3d.visualization.draw_geometries(all_meshes)

if __name__ == "__main__":
    main()


"""Used to just test Delaunay triangulation
"""
import numpy as np
from polylidar import Delaunator, MatrixDouble
import matplotlib.pyplot as plt

def main():
    points = np.random.randn(1000, 2)

    # Use Delaunay Triangulation
    points_mat = MatrixDouble(points)
    delaunay = Delaunator(points_mat)
    delaunay.triangulate()
    triangles = np.copy(np.asarray(delaunay.triangles))

    plt.triplot(points[:,0], points[:,1], triangles)

    plt.scatter(points[:, 0], points[: ,1])
    plt.show()

if __name__ == "__main__":
    main()


from os import path
import pytest
import numpy as np


from tests.python.helpers.utils import load_csv, load_npy
from polylidar.polylidarutil import generate_test_points, get_estimated_lmax

np.random.seed(1)


@pytest.fixture
def building1():
    return load_csv('building1.csv')


@pytest.fixture
def building2():
    return load_csv("building2.csv")


@pytest.fixture
def hardcase1():
    return load_csv("hardcase1.csv")


# @pytest.fixture
# def bad_convex_hull():
#     return load_npy("possible_error_free.npy")


@pytest.fixture()
def basic_params():
    return dict(alpha=0.5)


@pytest.fixture()
def params_lmax():
    return dict(alpha=0.0, lmax=100.0)


@pytest.fixture()
def hardcase1_params():
    return dict(alpha=0.0, lmax=20.0)


@pytest.fixture
def np_100K_array():
    return load_csv("100K_array_2d.csv")


@pytest.fixture
def np_100K_array_3d():
    return load_csv("100K_array_3d.csv")


@pytest.fixture()
def bad_convex_hull_params():
    return dict(alpha=0.0, lmax=1300.0)

@pytest.fixture(params=range(2, 11))
def num_groups(request):
    return request.param

@pytest.fixture(params=[1000, 10000])
def group_size(request):
    return request.param

@pytest.fixture
def cluster_groups(num_groups, group_size):
    cluster_params = dict(num_groups=num_groups, group_size=group_size)
    points = generate_test_points(**cluster_params)
    lmax = get_estimated_lmax(**cluster_params)
    polylidar_kwargs = dict(alpha=0.0, lmax=lmax)
    return dict(points=points, polylidar_kwargs=polylidar_kwargs, cluster_params=cluster_params)

# This creates 100 numpy arrays frangin from (1000,2) -> (100000,2)
ts = range(1000, 100000, 1000)
@pytest.fixture(params=ts)
def random_points(request):
    points = np.random.randn(request.param, 2) * 100 + 700000
    return points



"""Demo of Open3D 0.10.0 Slowdown
Please modify DIRECTORY to point to the folder of meshes attached to this issue reply
"""
import os
import open3d as o3d
import copy
DIRECTORY = 'fixtures/o3d_slow_down'
# o3d 0.10.0 - 9 Seconds to load meshes (time to being user interation), 1 FPS (with draw edges enabled 'w')
# o3d 0.11.0+f1d478c4 - 0.5 seconds, 45-60 FPS (with draw edges)

duplicate = 50
def main():
    all_meshes = []
    all_files = sorted(list(os.listdir(DIRECTORY)))
    for filename in all_files:
        print(filename)
        mesh = o3d.io.read_triangle_mesh(os.path.join(DIRECTORY, filename))
        
        all_meshes.extend([ copy.deepcopy(mesh) for i in range(duplicate)])
    print(len(all_meshes))
    o3d.visualization.draw_geometries(all_meshes)

if __name__ == "__main__":
    main()


import time
import logging
import sys
import functools
import operator

import numpy as np
import open3d as o3d
import matplotlib.cm as cm
import yaml


from polylidar import (Polylidar3D, MatrixDouble, MatrixFloat, extract_tri_mesh_from_float_depth,
                       extract_point_cloud_from_float_depth, MatrixUInt8)

from polylidar.polylidarutil.open3d_util import construct_grid, create_lines, flatten, create_open_3d_mesh_from_tri_mesh
from polylidar.polylidarutil.plane_filtering import filter_planes_and_holes
from polylidar.polylidarutil.line_mesh import LineMesh

from organizedpointfilters.utility.helper import create_meshes_cuda
from fastga import GaussianAccumulatorS2Beta, IcoCharts

from examples.python.util.helper_polylidar import extract_all_dominant_plane_normals, extract_planes_and_polygons_from_mesh, extract_planes_and_polygons_from_classified_mesh
from examples.python.util.helper_mesh import create_open_3d_mesh_from_tri_mesh


def generate_planar_pointcloud(bounds_x=[0, 64, 1], bounds_y=[0, 64, 1], z=10, z_noise=0.03):
    X, Y = np.mgrid[bounds_x[0]:bounds_x[1]:bounds_x[2], bounds_y[0]:bounds_y[1]:bounds_y[2]]
    pc = np.column_stack((X.ravel(), Y.ravel()))

    lateral_noise_amt = bounds_x[2] / 6
    noise = np.random.randn(*pc.shape) * lateral_noise_amt
    pc = pc + noise

    z = np.ones((pc.shape[0])) * z + np.random.randn(pc.shape[0]) * z_noise
    pc = np.column_stack([pc[:, 0], pc[:, 1], z])
    return pc


def convert_to_square_image(pc, invert=False):
    if invert:
        row = int(pc.shape[0])
        col = int(pc.shape[1])
        new_pc = pc.reshape((row * col, 3))
    else:
        row = int(np.sqrt(pc.shape[0]))
        new_shape = (row, row, 3)
        new_pc = pc.reshape(new_shape)
    return new_pc


def classify_point_cloud(pc):
    row = int(np.sqrt(pc.shape[0]))
    new_shape = (row, row)
    classes = np.ones(new_shape, dtype=np.uint8)

    classes = change_class(classes)

    return classes


def change_class(classes):
    percent = 0.25
    start_percent = 0.5 - percent / 2.0
    end_percent = 0.5 + percent / 2.0
    start = int(classes.shape[0] * start_percent)
    end = int(classes.shape[0] * end_percent)
    classes[start:end, start:end] = 0
    return classes

def get_np_buffer_ptr(a):
    pointer, read_only_flag = a.__array_interface__['data']
    return pointer

def create_color(classes):
    classes_ = (classes / 1.0)
    colors = cm.viridis(classes_)
    colors = colors[:, :, :3]
    colors = np.copy(convert_to_square_image(colors, True))
    return colors

def extract_polygons(opc, classes, config):
    pl = Polylidar3D(**config['polylidar'])
    ga = GaussianAccumulatorS2Beta(level=config['fastga']['level'])
    ico = IcoCharts(level=config['fastga']['level'])
    # 1. Create mesh
    tri_mesh = create_meshes_cuda(
        opc, **config['mesh']['filter'])

    mesh_o3d = create_open_3d_mesh_from_tri_mesh(tri_mesh)
    classes = classes.reshape((np.asarray(mesh_o3d.vertices).shape[0], 1))
    classes_mat = MatrixUInt8(classes)
    tri_mesh.set_vertex_classes(classes_mat, True)
    # alg_timings.update(timings)
    # 2. Get dominant plane normals
    avg_peaks, _, _, _, alg_timings = extract_all_dominant_plane_normals(
        tri_mesh, ga_=ga, ico_chart_=ico, **config['fastga'])

    # alg_timings.update(timings)
    # 3. Extract Planes and Polygons
    planes, obstacles, timings = extract_planes_and_polygons_from_classified_mesh(tri_mesh, avg_peaks, pl_=pl,
                                                                        filter_polygons=True, optimized=True,
                                                                        postprocess=config['polygon']['postprocess'])

    return mesh_o3d, planes, obstacles, timings

def handle_shapes(planes, obstacles, line_radius=0.15):
    all_polys = []

    for plane, _ in planes:
        points = np.array(plane.exterior)
        line_mesh = LineMesh(points, colors=[0,1,0], radius=line_radius)
        all_polys.append(line_mesh)

    for plane, _ in obstacles:
        points = np.array(plane.exterior)
        line_mesh = LineMesh(points, colors=[1, 0, 0], radius=line_radius)
        all_polys.append(line_mesh)

    all_meshes = [poly.cylinder_segments for poly in all_polys]
    all_meshes = functools.reduce(operator.iconcat, all_meshes, [])

    return all_polys, all_meshes

def main():
    # Load yaml file
    config = None
    with open('./examples/python/for_landing/PolylidarParamsSimple.yaml') as file:
        try:
            config = yaml.safe_load(file)
        except yaml.YAMLError as exc:
            print("Error parsing yaml")

    pc = generate_planar_pointcloud()
    pc_image = convert_to_square_image(pc)
    classes = classify_point_cloud(pc)
    colors = create_color(classes)

    pcd = o3d.geometry.PointCloud(o3d.utility.Vector3dVector(pc))
    pcd.colors = o3d.utility.Vector3dVector(colors)
    o3d.visualization.draw_geometries([pcd])


    mesh_o3d, planes, obstacles, time = extract_polygons(pc_image, classes, config)

    all_polys, all_meshes = handle_shapes(planes, obstacles)
    all_meshes = [mesh_o3d, *all_meshes]
    # for i, mesh in enumerate(all_meshes):
    #     o3d.io.write_triangle_mesh(f"o3d_slow_down/mesh_test_{i:02d}.ply", mesh)
    o3d.visualization.draw_geometries([mesh_o3d, pcd, *all_meshes])

    


if __name__ == "__main__":
    main()



from os import path
import pytest
import numpy as np

from shapely.geometry import Polygon
from polylidar import Polylidar3D, MatrixDouble, HalfEdgeTriangulation

DIR_NAME = path.dirname(path.dirname(path.dirname(path.dirname(__file__))))
FIXTURES_DIR = path.join(DIR_NAME, 'fixtures', 'pointsets')

def get_poly_coords(outline, points):
    return [get_point(pi, points) for pi in outline]

def get_point(pi, points):
    return [points[pi, 0], points[pi, 1]]

def load_csv(file, delimeter=','):
    return np.loadtxt(path.join(FIXTURES_DIR, file), delimiter=delimeter, dtype=np.float64, skiprows=1)

def load_npy(file):
    return np.load(path.join(FIXTURES_DIR, file))

def verify_points(data, n_cols=2):
    assert isinstance(data, np.ndarray)
    assert data.shape[1] == n_cols

def basic_polylidar_verification(points, mesh:HalfEdgeTriangulation, planes, polygons):
    # make sure delaunay has the correct number of points
    assert points.shape[0] == np.array(mesh.vertices).shape[0]
    # make sure that some planes were extracted
    assert len(planes) > 0
    # make sure that some polygons were extracted
    assert len(polygons) > 0

def verify_all_polygons_are_valid(polygons, points):
    for poly in polygons:
        verify_valid_polygon(poly, points)

def verify_valid_polygon(poly, points):
    shell_coords = get_poly_coords(poly.shell, points)
    hole_coords = [get_poly_coords(hole, points) for hole in poly.holes]
    # Verify polygon outer shell
    shapelyPoly = Polygon(shell=shell_coords)
    assert shapelyPoly.is_valid
    # Verify polygon outer shell and holes
    shapelyPoly = Polygon(shell=shell_coords, holes=hole_coords)
#     if not shapelyPoly.is_valid:
#         np.save("scratch/error_{}.npy".format(points.shape[0]), points)
    assert shapelyPoly.is_valid

def verify_all(points, polylidar_kwargs):
    pl = Polylidar3D(**polylidar_kwargs)
    points_mat = MatrixDouble(points)
    mesh, planes, polygons = pl.extract_planes_and_polygons(points_mat)
    # Basic test to ensure no obvious errors occurred
    basic_polylidar_verification(points, mesh, planes, polygons)
    # Ensure that the polygons returned are valid
    verify_all_polygons_are_valid(polygons, points)






from setuptools import setup, find_packages
import glob

# Force platform specific wheel
try:
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
    # https://stackoverflow.com/a/45150383/1255535

    class bdist_wheel(_bdist_wheel):
        def finalize_options(self):
            _bdist_wheel.finalize_options(self)
            self.root_is_pure = False
    print("bdist wheel is working and installed")
except ImportError:
    print('Warning: cannot import "wheel" package to build platform-specific wheel')
    print('Install the "wheel" package to fix this warning')
    bdist_wheel = None

cmdclass = {'bdist_wheel': bdist_wheel} if bdist_wheel is not None else dict()


# Read requirements.txt
with open('requirements.txt', 'r') as f:
    lines = f.readlines()
install_requires = [line.strip() for line in lines if line]


setup(
    author='Jeremy Castagno',
    author_email='',
    classifiers=[
    ],
    description=[
        "Polylidar ...."
    ],
    cmdclass=cmdclass,
    install_requires=install_requires,
    include_package_data=True,
    keywords="",
    license="MIT",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    # Name of the package on PyPI
    name="@PYPI_PACKAGE_NAME@",
    packages=[
        'polylidar',
    ],
    url="@PROJECT_HOME@",
    project_urls={
        'Documentation': '@PROJECT_DOCS@',
        'Source code': '@PROJECT_CODE@',
        'Issues': '@PROJECT_ISSUES@',
    },
    version='@PROJECT_VERSION@',
    zip_safe=False,
)



import time
import logging
import warnings
import numpy as np
from copy import deepcopy
from scipy.spatial.transform import Rotation as R
import matplotlib.pyplot as plt

warnings.filterwarnings("ignore", message="Optimal rotation is not uniquely or poorly defined ")
np.set_printoptions(precision=4, suppress=True)

from examples.python.util.realsense_util import (get_realsense_data, get_frame_data, R_Standard_d400, prep_mesh,
                                            create_open3d_pc, extract_mesh_planes, COLOR_PALETTE, create_open_3d_mesh)
from examples.python.util.mesh_util import get_mesh_data_iterator

from polylidar import (Polylidar3D, MatrixDouble, MatrixFloat, MatrixInt, create_tri_mesh_copy, bilateral_filter_normals)

from polylidar.polylidarutil.open3d_util import construct_grid, create_lines, flatten
from polylidar.polylidarutil.plane_filtering import filter_planes_and_holes

from fastga import GaussianAccumulatorS2, MatX3d
from fastga.peak_and_cluster import find_peaks_from_accumulator


import open3d as o3d


def filter_and_create_open3d_polygons(points, polygons, rm=None, line_radius=0.005):
    " Apply polygon filtering algorithm, return Open3D Mesh Lines "
    config_pp = dict(filter=dict(hole_area=dict(min=0.1, max=100.0), hole_vertices=dict(min=6), plane_area=dict(min=0.25)),
                     positive_buffer=0.02, negative_buffer=0.05, simplify=0.02)
    # config_pp = dict(filter=dict(hole_area=dict(min=0.00, max=100.0), hole_vertices=dict(min=3), plane_area=dict(min=0.05)),
    #                  positive_buffer=0.00, negative_buffer=0.0, simplify=0.01)
    t1 = time.perf_counter()
    planes, obstacles = filter_planes_and_holes(polygons, points, config_pp, rm=rm)
    t2 = time.perf_counter()
    logging.info("Plane Filtering Took (ms): %.2f", (t2-t1) * 1000)
    all_poly_lines = create_lines(planes, obstacles, line_radius=line_radius)
    return all_poly_lines, (t2-t1) * 1000

def open_3d_mesh_to_trimesh(mesh: o3d.geometry.TriangleMesh):
    triangles = np.asarray(mesh.triangles)
    vertices = np.asarray(mesh.vertices)
    triangles = np.ascontiguousarray(triangles)
    vertices_mat = MatrixDouble(vertices)
    triangles_mat = MatrixInt(triangles)
    tri_mesh = create_tri_mesh_copy(vertices_mat, triangles_mat)
    return tri_mesh

def extract_all_dominant_planes(tri_mesh, vertices, polylidar_kwargs, ds=50, min_samples=10000):
    ga = GaussianAccumulatorS2(level=4, max_phi=180)
    triangle_normals = np.asarray(tri_mesh.triangle_normals)
    num_normals = triangle_normals.shape[0]

    # Downsample, TODO improve this
    ds_normals = int(num_normals / ds)
    to_sample = max(min([num_normals, min_samples]), ds_normals)
    ds_step = int(num_normals / to_sample)

    triangle_normals_ds = np.ascontiguousarray(triangle_normals[:num_normals:ds_step, :])
    # A copy occurs here for triangle_normals......if done in c++ there would be no copy
    ga.integrate(MatX3d(triangle_normals_ds))
    gaussian_normals = np.asarray(ga.get_bucket_normals())
    accumulator_counts = np.asarray(ga.get_normalized_bucket_counts())
    _, _, avg_peaks, _ = find_peaks_from_accumulator(gaussian_normals, accumulator_counts)
    logging.info("Processing mesh with %d triangles", num_normals)
    logging.info("Dominant Plane Normals")
    print(avg_peaks)

    avg_peaks_selected = np.copy(avg_peaks[[0, 1, 2, 3], :])
    pl = Polylidar3D(**polylidar_kwargs)
    avg_peaks_mat = MatrixDouble(avg_peaks_selected)

    tri_set = pl.extract_tri_set(tri_mesh, avg_peaks_mat)
    t0 = time.perf_counter()

    all_planes, all_polygons = pl.extract_planes_and_polygons_optimized(tri_mesh, avg_peaks_mat)
    t1 = time.perf_counter()
    polylidar_time = (t1-t0) * 1000

    all_poly_lines = []
    for i in range(avg_peaks_selected.shape[0]):
        avg_peak = avg_peaks[i, :]
        rm, _ = R.align_vectors([[0, 0, 1]], [avg_peak])
        polygons_for_normal = all_polygons[i]
        # print(polygons_for_normal)
        if len(polygons_for_normal) > 0:
            poly_lines, _ = filter_and_create_open3d_polygons(vertices, polygons_for_normal, rm=rm)
            all_poly_lines.extend(poly_lines)

    return all_planes, tri_set, all_poly_lines, polylidar_time

def split_triangles(mesh):
    """
    Split the mesh in independent triangles    
    """
    triangles = np.asarray(mesh.triangles).copy()
    vertices = np.asarray(mesh.vertices).copy()

    triangles_3 = np.zeros_like(triangles)
    vertices_3 = np.zeros((len(triangles) * 3, 3), dtype=vertices.dtype)

    for index_triangle, t in enumerate(triangles):
        index_vertex = index_triangle * 3
        vertices_3[index_vertex] = vertices[t[0]]
        vertices_3[index_vertex + 1] = vertices[t[1]]
        vertices_3[index_vertex + 2] = vertices[t[2]]

        triangles_3[index_triangle] = np.arange(index_vertex, index_vertex + 3)

    mesh_return = deepcopy(mesh)
    mesh_return.triangles = o3d.utility.Vector3iVector(triangles_3)
    mesh_return.vertices = o3d.utility.Vector3dVector(vertices_3)
    mesh_return.triangle_normals = mesh.triangle_normals
    mesh_return.paint_uniform_color([0.5, 0.5, 0.5])
    return mesh_return

def assign_some_vertex_colors(mesh, triangle_indices, triangle_colors, mask=None):
    """Assigns vertex colors by given normal colors
    NOTE: New mesh is returned

    Arguments:
        mesh {o3d:TriangleMesh} -- Mesh
        normal_colors {ndarray} -- Normals Colors

    Returns:
        o3d:TriangleMesh -- New Mesh with painted colors
    """
    split_mesh = split_triangles(mesh)
    vertex_colors = np.asarray(split_mesh.vertex_colors)
    triangles = np.asarray(split_mesh.triangles)
    if mask is not None:
        triangles = triangles[mask, :]

    if isinstance(triangle_indices, list):
        for triangle_set, color in zip(triangle_indices, triangle_colors):
            triangle_set = np.asarray(triangle_set)
            for i in range(np.asarray(triangle_set).shape[0]):
                # import ipdb; ipdb.set_trace()
                t_idx = triangle_set[i]
                p_idx = triangles[t_idx, :]
                vertex_colors[p_idx] = color
    else:
        for i in range(triangle_indices.shape[0]):
            # import ipdb; ipdb.set_trace()
            t_idx = triangle_indices[i]
            color = triangle_colors[i, :]
            p_idx = triangles[t_idx, :]
            vertex_colors[p_idx] = color
    if not split_mesh.has_triangle_normals():
        split_mesh.compute_triangle_normals()
    split_mesh.compute_vertex_normals()

    return split_mesh

def paint_planes(o3d_mesh, planes):
    # colors = np.arange(0, 0+ len(planes))
    colors = [0, 3]
    all_colors = plt.cm.get_cmap('tab10')(colors)[:, :3]

    # planes_list = [np.copy(plane) for plane in planes]
    # planes_list = np.

    new_mesh = assign_some_vertex_colors(o3d_mesh, planes, all_colors)
    return new_mesh

def run_test(mesh, callback=None, stride=2):
    # Create Pseudo 3D Surface Mesh using Delaunay Triangulation and Polylidar
    polylidar_kwargs = dict(alpha=0.0, lmax=0.10, min_triangles=1000,
                            z_thresh=0.06, norm_thresh=0.95, norm_thresh_min=0.95, min_hole_vertices=6)
    # Create Polylidar TriMesh
    tri_mesh = open_3d_mesh_to_trimesh(mesh)
    bilateral_filter_normals(tri_mesh, 3, 0.1, 0.1)
    vertices = np.asarray(tri_mesh.vertices)
    normals_smooth = np.asarray(tri_mesh.triangle_normals)
    mesh.triangle_normals = o3d.utility.Vector3dVector(normals_smooth)

    o3d.visualization.draw_geometries([mesh], width=600, height=500)

    planes, tri_set, all_poly_lines, polylidar_time = extract_all_dominant_planes(tri_mesh, vertices, polylidar_kwargs)
    time_polylidar3D = polylidar_time
    polylidar_3d_alg_name = 'Polylidar3D with Provided Mesh'
    
    # planes_tri_set = [np.argwhere(np.asarray(tri_set) == i)  for i in range(1, 3)]
    # # import ipdb; ipdb.set_trace()
    # mesh_tri_set = paint_planes(mesh, planes_tri_set)
    # callback(polylidar_3d_alg_name, time_polylidar3D, mesh_tri_set)

    # mesh_segment = paint_planes(mesh, planes)
    # callback(polylidar_3d_alg_name, time_polylidar3D, mesh_segment)


    mesh_3d_polylidar = []
    mesh_3d_polylidar.extend(flatten([line_mesh.cylinder_segments for line_mesh in all_poly_lines]))
    mesh_3d_polylidar.append(mesh)
    callback(polylidar_3d_alg_name, time_polylidar3D, mesh_3d_polylidar)

def callback(alg_name, execution_time,mesh=None):
    axis_frame = o3d.geometry.TriangleMesh.create_coordinate_frame(size=0.2)
    axis_frame.translate([0, 0.8, -0.7])
    grid_ls = construct_grid(size=2, n=20, plane_offset=-0.8, translate=[0, 1.0, 0.0])
    logging.info("%s took (ms): %.2f", alg_name, execution_time)
    if mesh:
        if isinstance(mesh, list):
            o3d.visualization.draw_geometries(
                [*mesh, axis_frame], width=600, height=500)
        else:
            o3d.visualization.draw_geometries([mesh, axis_frame], width=600, height=500)

def main():
    axis_frame = o3d.geometry.TriangleMesh.create_coordinate_frame(size=0.2)
    axis_frame.translate([0, 0.8, -1.0])
    grid_ls = construct_grid(size=2, n=20, plane_offset=-1.0, translate=[0, 0.0, 0.0])
    for i, mesh in enumerate(get_mesh_data_iterator()):
        if i < 2:
            continue
            # o3d.io.write_triangle_mesh('test.ply', mesh)
        
        run_test(mesh, callback=callback, stride=2)


if __name__ == "__main__":
    main()




"""
{
	"class_name" : "ViewTrajectory",
	"interval" : 29,
	"is_loop" : false,
	"trajectory" : 
	[
		{
			"boundingbox_max" : [ 1.8801286101866053, 3.0496817724620651, 3.0527941421471674 ],
			"boundingbox_min" : [ -2.250705786558127, -3.7003182275379354, -0.71199999999999997 ],
			"field_of_view" : 60.0,
			"front" : [ -0.64444596129317422, 0.60835840406357145, 0.46323801137011755 ],
			"lookat" : [ 0.14706315217196639, -0.78866172526659861, 1.3194472369723524 ],
			"up" : [ 0.3610685397047429, -0.29191354924403706, 0.88567261976603617 ],
			"zoom" : 0.19999999999999959
		}
	],
	"version_major" : 1,
	"version_minor" : 0
}
"""



import time
import math

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import matplotlib 

matplotlib.rc('xtick', labelsize=14) 
matplotlib.rc('ytick', labelsize=14)
# matplotlib.rc('ztick', labelsize=20)
matplotlib.rc('axes', labelsize=18) 


from polylidar import Delaunator, MatrixDouble, Polylidar3D
from polylidar.polylidarutil import (plot_polygons_3d, generate_3d_plane, set_axes_equal, plot_planes_3d,
                           scale_points, rotation_matrix, apply_rotation)



def set_labels(ax):
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')


def main():
    np.random.seed(1)
    # generate random plane with hole
    plane = generate_3d_plane(bounds_x=[0, 10, 0.5], bounds_y=[0, 10, 0.5], holes=[
                            [[3, 6], [3, 6]]], height_noise=0.04, planar_noise=0.04)

    large_wall = generate_3d_plane(bounds_x=[0.5, 4, 1.0], bounds_y=[0, 9.5, 0.5], holes=[], height_noise=0.04, planar_noise=0.1)
    # Generate top of box (causing the hole that we see)
    box_top = generate_3d_plane(bounds_x=[3, 6, 0.5], bounds_y=[3, 6, 0.5], holes=[
    ], height_noise=0.02, height=2, planar_noise=0.03)
    # Generate side of box (causing the hole that we see)
    box_side = generate_3d_plane(bounds_x=[.25, 2, 0.5], bounds_y=[
                                0, 3, 0.5], holes=[], height_noise=0.02, planar_noise=0.02)
    rm = rotation_matrix([0,1,0], -math.pi/2.0)
    box_side = apply_rotation(rm, box_side) + [6, 3, 0]

    large_wall = apply_rotation(rm , large_wall) + [10, 0, -3.5]
    # box_side = r.apply(box_side) + [5, 3, 0]
    # All points joined together
    points = np.concatenate((plane, box_side, box_top, large_wall))

    points_mat = MatrixDouble(points)
    polylidar_kwargs = dict(alpha=0.0, lmax=1.0, min_triangles=10, z_thresh=0.15, norm_thresh_min=0.95)
    polylidar = Polylidar3D(**polylidar_kwargs)

    elev = 15.0
    azim = -35

    # Show Point Cloud
    print("Should see point raw point cloud")
    fig, ax = plt.subplots(figsize=(10, 10), nrows=1, ncols=1,
                    subplot_kw=dict(projection='3d'))
    # plot points
    ax.scatter(*scale_points(points), s=20.0, c=points[:, 2], cmap=plt.cm.plasma)
    set_axes_equal(ax, ignore_z=True)
    ax.set_zlim3d([-4, 6])
    ax.view_init(elev=elev, azim=azim)
    set_labels(ax)
    fig.savefig("assets/scratch/Basic25DAlgorithm_pointcloud.pdf", bbox_inches='tight')
    fig.savefig("assets/scratch/Basic25DAlgorithm_pointcloud.png", bbox_inches='tight', pad_inches=-0.8)
    plt.show()


    # Extracts planes and polygons, time
    t1 = time.time()
    mesh, planes, polygons = polylidar.extract_planes_and_polygons(points_mat)
    t2 = time.time()
    print("Polylidar Took {:.2f} milliseconds".format((t2 - t1) * 1000))

    triangles = np.asarray(mesh.triangles)
    all_planes = [np.arange(triangles.shape[0])]

    # Show Triangulation
    fig, ax = plt.subplots(figsize=(10, 10), nrows=1, ncols=1,
                    subplot_kw=dict(projection='3d'))

    plot_planes_3d(points, triangles, all_planes, ax, alpha=0.0, z_value=-4.0)
    plot_planes_3d(points, triangles, all_planes, ax, alpha=0.5)
    # plot points
    ax.scatter(*scale_points(points), s=0.1, c='k')
    set_axes_equal(ax, ignore_z=True)
    ax.set_zlim3d([-4, 6])
    ax.view_init(elev=elev, azim=azim)
    set_labels(ax)
    print("Should see triangulation point cloud")
    fig.savefig("assets/scratch/Basic25DAlgorithm_mesh.pdf", bbox_inches='tight')
    fig.savefig("assets/scratch/Basic25DAlgorithm_mesh.png", bbox_inches='tight', pad_inches=-0.8)
    plt.show()


    
    print("")
    print("Should see two planes extracted, please rotate.")
    fig, ax = plt.subplots(figsize=(10, 10), nrows=1, ncols=1,
                        subplot_kw=dict(projection='3d'))
    # plot all triangles
    plot_polygons_3d(points, polygons, ax)
    plot_planes_3d(points, triangles, planes, ax)
    # plot points
    ax.scatter(*scale_points(points), c='k', s=0.1)
    set_axes_equal(ax, ignore_z=True)
    ax.set_zlim3d([-4, 6])
    ax.view_init(elev=elev, azim=azim)
    set_labels(ax)
    fig.savefig("assets/scratch/Basic25DAlgorithm_polygons.pdf", bbox_inches='tight')
    fig.savefig("assets/scratch/Basic25DAlgorithm_polygons.png", bbox_inches='tight', pad_inches=-0.8)
    plt.show()
    print("")

if __name__ == "__main__":
    main()



from os import path
import pytest
import numpy as np


from tests.python.helpers.utils import load_csv, verify_points, basic_polylidar_verification, verify_all_polygons_are_valid, load_npy
try:
    from polylidar import extractPlanesAndPolygons, extractPolygons, extractPolygonsAndTimings
    Polylidar3D = None
except Exception:
    print("Using new Polylidar3D")
    from polylidar import Polylidar3D, MatrixDouble


# def test_building1(benchmark, building1, basic_params):
#     benchmark(extractPlanesAndPolygons, building1, **basic_params)


# def test_building2(benchmark, building2, basic_params):
#     benchmark(extractPlanesAndPolygons, building2, **basic_params)


def test_100k_array_lmax(benchmark, np_100K_array, params_lmax):
    if Polylidar3D is not None:
        points_mat = MatrixDouble(np_100K_array)
        pl = Polylidar3D(**params_lmax)
        _, _ , _  = benchmark(pl.extract_planes_and_polygons, points_mat)
    else:
        benchmark(extractPolygons, np_100K_array, **params_lmax)

def test_100k_array_3d_lmax(benchmark, np_100K_array_3d, params_lmax):
    if Polylidar3D is not None:
        points_mat = MatrixDouble(np_100K_array_3d)
        pl = Polylidar3D(**params_lmax)
        _, _ , _  = benchmark(pl.extract_planes_and_polygons, points_mat)
    else:
        benchmark(extractPolygons, np_100K_array_3d, **params_lmax)

def test_clusters(benchmark, cluster_groups):
    """
    Will benchmark clusters AND check that python overhead is less than 3ms (ussually <1 ms)
    """
    points = cluster_groups['points']
    polylidar_kwargs = cluster_groups['polylidar_kwargs']
    if Polylidar3D is not None:
        points_mat = MatrixDouble(points)
        pl = Polylidar3D(**polylidar_kwargs)
        _, _ , _  = benchmark(pl.extract_planes_and_polygons, points_mat)
    else:
        _, _ , _  = benchmark(extractPlanesAndPolygons, points, **polylidar_kwargs)


# def test_clusters(benchmark, cluster_groups):
#     """
#     Will benchmark clusters AND check that python overhead is less than 3ms (ussually <1 ms)
#     """
#     points = cluster_groups['points']
#     polylidar_kwargs = cluster_groups['polylidar_kwargs']
#     _, timings = benchmark(extractPolygonsAndTimings, points, **polylidar_kwargs)
#     cpp_timing = np.sum(timings) / 1000.0 # Actuall c++ timing
#     median_benchmark = benchmark.stats.stats.median # python benchmark timing
#     assert cpp_timing == pytest.approx(median_benchmark, abs=0.003)
    



# -*- coding: utf-8 -*-
import os
import sys
import subprocess

from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext

# Convert distutils Windows platform specifiers to CMake -A arguments
PLAT_TO_CMAKE = {
    "win32": "Win32",
    "win-amd64": "x64",
    "win-arm32": "ARM",
    "win-arm64": "ARM64",
}


# A CMakeExtension needs a sourcedir instead of a file list.
class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


# Read requirements.txt
with open('requirements.txt', 'r') as f:
    lines = f.readlines()
install_requires = [line.strip() for line in lines if line]

# Read Version
with open('src/version.txt', 'r') as f:
    lines = f.readlines()
print(lines)
version = ".".join([line.split(' ')[1].strip() for line in lines if line])
print("The version is {}".format(version))

class CMakeBuild(build_ext):
    def run(self):
        # This is optional - will print a nicer error if CMake is missing.
        # Since we force CMake via PEP 518 in the pyproject.toml, this should
        # never happen and this whole method can be removed in your code if you
        # want.
        try:
            subprocess.check_output(["cmake", "--version"])
        except OSError:
            msg = "CMake missing - probably upgrade to a newer version of Pip?"
            raise RuntimeError(msg)

        # To support Python 2, we have to avoid super(), since distutils is all
        # old-style classes.
        build_ext.run(self)

    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))

        # required for auto-detection of auxiliary "native" libs
        if not extdir.endswith(os.path.sep):
            extdir += os.path.sep

        cfg = "Debug" if self.debug else "Release"

        # CMake lets you override the generator - we need to check this.
        # Can be set with Conda-Build, for example.
        cmake_generator = os.environ.get("CMAKE_GENERATOR", "")

        print(cfg)
        print(extdir)
        print(sys.executable)
        

        # Set Python_EXECUTABLE instead if you use PYBIND11_FINDPYTHON
        # EXAMPLE_VERSION_INFO shows you how to pass a value into the C++ code
        # from Python.
        cmake_args = [
            "-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={}".format(extdir),
            "-DPYTHON_EXECUTABLE={}".format(sys.executable),
            "-DCMAKE_BUILD_TYPE={}".format(cfg),  # not used on MSVC, but no harm
            "-DPL_BUILD_TESTS=OFF",
            "-DPL_BUILD_BENCHMARKS=OFF",
            "-DPL_BUILD_EXAMPLES=OFF",
        ]
        build_args = []

        if self.compiler.compiler_type != "msvc":
            # Using Ninja-build since it a) is available as a wheel and b)
            # multithreads automatically. MSVC would require all variables be
            # exported for Ninja to pick it up, which is a little tricky to do.
            # Users can override the generator with CMAKE_GENERATOR in CMake
            # 3.15+.
            pass
            # if not cmake_generator:
            #     cmake_args += ["-GNinja"]

        else:

            # Single config generators are handled "normally"
            single_config = any(x in cmake_generator for x in {"NMake", "Ninja"})

            # CMake allows an arch-in-generator style for backward compatibility
            contains_arch = any(x in cmake_generator for x in {"ARM", "Win64"})

            # Specify the arch if using MSVC generator, but only if it doesn't
            # contain a backward-compatibility arch spec already in the
            # generator name.
            if not single_config and not contains_arch:
                cmake_args += ["-A", PLAT_TO_CMAKE[self.plat_name]]

            # Multi-config generators have a different way to specify configs
            if not single_config:
                cmake_args += [
                    "-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}".format(cfg.upper(), extdir)
                ]
                build_args += ["--config", cfg]

        # Set CMAKE_BUILD_PARALLEL_LEVEL to control the parallel build level
        # across all generators.
        if "CMAKE_BUILD_PARALLEL_LEVEL" not in os.environ:
            # self.parallel is a Python 3 only way to set parallel jobs by hand
            # using -j in the build_ext call, not supported by pip or PyPA-build.
            if hasattr(self, "parallel") and self.parallel:
                # CMake 3.12+ only.
                build_args += ["-j{}".format(self.parallel)]
            else:
                build_args += ["-j4"]

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)

        subprocess.check_call(
            ["cmake", ext.sourcedir] + cmake_args, cwd=self.build_temp
        )
        subprocess.check_call(
            ["cmake", "--build", ".", "--target",  "python-package"] + build_args, cwd=self.build_temp
        )


setup(
    name="polylidar",
    version=version,
    author="Jeremy Castagno",
    author_email="jeremybyu@gmail.com",
    description="Polygon extracton from 3D Data",
    url='https://github.com/JeremyBYU/polylidar',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    license="MIT",
    include_package_data=True,
    packages=['polylidar', 'polylidar.polylidarutil'],
    package_dir={'':'src/Python'},
    ext_modules=[CMakeExtension("polylidar_pybind")],
    install_requires=install_requires,
    cmdclass={"build_ext": CMakeBuild},
    zip_safe=False,
)


"""
Simple example demonstrating a bilateral filter implented in C++.
Note that this is NOT the accelerated bilateral filter discussed in the Paper. This is just something fun I tried out that works for generalized meshes.
The accelerated bilateral filter for **organized** point clouds is found in OrganizedPointFilters repo (not this repo).
Requires: Fixtures data
"""
import time
import logging
import warnings

import numpy as np
from scipy.spatial.transform import Rotation as R
import open3d as o3d


from examples.python.util.mesh_util import get_mesh_data_iterator
from polylidar import bilateral_filter_normals
from polylidar.polylidarutil.open3d_util import open_3d_mesh_to_trimesh

def main():
    for i, mesh in enumerate(get_mesh_data_iterator()):
        if i < 0:
            continue
        mesh.compute_vertex_normals()
        mesh.compute_triangle_normals()
        print("Before")
        o3d.visualization.draw_geometries([mesh])

        tri_mesh = open_3d_mesh_to_trimesh(mesh)
        t1 = time.perf_counter()
        bilateral_filter_normals(tri_mesh, iterations=20, sigma_length=0.1, sigma_angle=0.1)
        t2 = time.perf_counter()
        print(t2-t1)
        normals_smooth = np.asarray(tri_mesh.triangle_normals)
        mesh.triangle_normals = o3d.utility.Vector3dVector(normals_smooth)

        print("After")
        o3d.visualization.draw_geometries([mesh])



if __name__ == "__main__":
    main()
